"use strict";
/*!
 * Copyright 2019 acrazing <joking.young@gmail.com>. All rights reserved.
 * @since 2019-03-27 17:49:03
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.flag = void 0;
const numberFormat_1 = require("./numberFormat");
/**
 * create flagged class fields, see flag.spec.ts
 * @param mask
 */
function flag(mask) {
    return (proto, key, desc) => {
        proto.__flag_mask__ = proto.__flag_mask__ || 0;
        if ((proto.__flag_mask__ & mask) !== 0) {
            throw new Error(`mask ${(0, numberFormat_1.itos)(mask, 2)} is used before set "${key}"`);
        }
        proto.__flag_mask__ |= mask;
        return {
            enumerable: true,
            configurable: true,
            get() {
                return (this.flags & mask) !== 0;
            },
            set(value) {
                if (value) {
                    this.flags |= mask;
                }
                else {
                    this.flags &= ~mask;
                }
                desc && desc.set && desc.set.call(this, value);
            },
        };
    };
}
exports.flag = flag;
//# sourceMappingURL=flag.js.map