/*!
 * Copyright 2019 acrazing <joking.young@gmail.com>. All rights reserved.
 * @since 2019-03-27 17:49:03
 */
import { itos } from './numberFormat';
/**
 * create flagged class fields, see flag.spec.ts
 * @param mask
 */
export function flag(mask) {
    return function (proto, key, desc) {
        proto.__flag_mask__ = proto.__flag_mask__ || 0;
        if ((proto.__flag_mask__ & mask) !== 0) {
            throw new Error("mask ".concat(itos(mask, 2), " is used before set \"").concat(key, "\""));
        }
        proto.__flag_mask__ |= mask;
        return {
            enumerable: true,
            configurable: true,
            get: function () {
                return (this.flags & mask) !== 0;
            },
            set: function (value) {
                if (value) {
                    this.flags |= mask;
                }
                else {
                    this.flags &= ~mask;
                }
                desc && desc.set && desc.set.call(this, value);
            },
        };
    };
}
//# sourceMappingURL=flag.js.map