"use strict";
/*!
 * Copyright 2019 acrazing <joking.young@gmail.com>. All rights reserved.
 * @since 2019-01-17 12:14:56
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.int2ip = exports.ip2int = exports.IP_MIN = exports.IP_MAX = void 0;
const net_1 = require("net");
const consts_1 = require("./consts");
exports.IP_MAX = '255.255.255.255';
exports.IP_MIN = '0.0.0.0';
/**
 * ipv4 string to integer
 * @param ip
 * @param defaults
 */
function ip2int(ip, defaults = consts_1.UINT32_MAX) {
    if (!(0, net_1.isIPv4)(ip)) {
        return defaults;
    }
    const [c0, c1, c2, c3] = ip.split('.');
    return +c0 * (1 << 24) + (+c1 << 16) + (+c2 << 8) + +c3;
}
exports.ip2int = ip2int;
/**
 * integer to ipv4 string
 * @param ipInt
 * @param defaults
 */
function int2ip(ipInt, defaults = exports.IP_MIN) {
    if (ipInt < 0 || ipInt > consts_1.UINT32_MAX) {
        return defaults;
    }
    return ((ipInt >>> 24) +
        '.' +
        ((ipInt >> 16) & 0xff) +
        '.' +
        ((ipInt >> 8) & 0xff) +
        '.' +
        (ipInt & 0xff));
}
exports.int2ip = int2ip;
//# sourceMappingURL=ip.js.map