/*!
 * Copyright 2019 acrazing <joking.young@gmail.com>. All rights reserved.
 * @since 2019-01-17 12:14:56
 */
import { __read } from "tslib";
import { isIPv4 } from 'net';
import { UINT32_MAX } from './consts';
export var IP_MAX = '255.255.255.255';
export var IP_MIN = '0.0.0.0';
/**
 * ipv4 string to integer
 * @param ip
 * @param defaults
 */
export function ip2int(ip, defaults) {
    if (defaults === void 0) { defaults = UINT32_MAX; }
    if (!isIPv4(ip)) {
        return defaults;
    }
    var _a = __read(ip.split('.'), 4), c0 = _a[0], c1 = _a[1], c2 = _a[2], c3 = _a[3];
    return +c0 * (1 << 24) + (+c1 << 16) + (+c2 << 8) + +c3;
}
/**
 * integer to ipv4 string
 * @param ipInt
 * @param defaults
 */
export function int2ip(ipInt, defaults) {
    if (defaults === void 0) { defaults = IP_MIN; }
    if (ipInt < 0 || ipInt > UINT32_MAX) {
        return defaults;
    }
    return ((ipInt >>> 24) +
        '.' +
        ((ipInt >> 16) & 0xff) +
        '.' +
        ((ipInt >> 8) & 0xff) +
        '.' +
        (ipInt & 0xff));
}
//# sourceMappingURL=ip.js.map