/*!
 *
 * Copyright 2017 - acrazing
 *
 * @author acrazing joking.young@gmail.com
 * @since 2017-12-15 12:29:38
 * @version 1.0.0
 * @desc nonenumerable.ts
 */
/**
 * create non-enumerable field
 * @param p
 * @param key
 * @param desc
 */
export function nonenumerable(p, key, desc) {
    desc = desc || Object.getOwnPropertyDescriptor(p, key);
    var pValue;
    if (desc) {
        desc.enumerable = false;
        var get_1 = desc.get, set_1 = desc.set;
        pValue = desc.value;
        desc.set = function (value) {
            set_1 && set_1.call(this, value);
            if (this === p) {
                pValue = value;
                return;
            }
            var instDesc = Object.getOwnPropertyDescriptor(this, key);
            if (instDesc) {
                instDesc.enumerable = false;
                Object.defineProperty(this, key, instDesc);
            }
            else {
                Object.defineProperty(this, key, {
                    configurable: true,
                    enumerable: false,
                    writable: true,
                    value: value,
                });
            }
        };
        desc.get = function () {
            return get_1 ? get_1.call(this) : pValue;
        };
        return desc;
    }
    return {
        enumerable: false,
        configurable: true,
        set: function (value) {
            if (this === p) {
                pValue = value;
                return;
            }
            Object.defineProperty(this, key, {
                enumerable: false,
                configurable: true,
                writable: true,
                value: value,
            });
        },
        get: function () {
            return pValue;
        },
    };
}
//# sourceMappingURL=nonenumerable.js.map